@@ -1,3 +1,5 @@
+# 2011-05-15 CJS limited etaU to 20 or less
+# 2011-01-24 SB  added call to run.windows.openbugs and run.windows.winbugs
 # 2010-11-20 CJS added code to display progress of sampling during burnin and posterior to the user
 # 2010-11-19 SB  add code to make initial U a minimum of 1 to prevent crashing
 # 2010-04-26 CJS fixed problem with init.logitP when n1=m2=k and you get +infinity which craps out lm()
@@ -7,17 +9,32 @@
 # 2009-12-05 CJS added title to argument list
 # 2009-12-01 CJS added openbugs/winbugs directory to argument list
 
-TimeStratPetersenNonDiagError <- function(title, prefix, time, n1, m2, u2,
-                     jump.after=NULL,
-                     logitP.cov=rep(1,length(u2)), logitP.fixed=rep(NA,length(u2)),
-                     n.chains=3, n.iter=200000, n.burnin=100000, n.sims=2000, 
-                     tauU.alpha=1, tauU.beta=.05, taueU.alpha=1, taueU.beta=.05,
-                     mu_xiP=-2, tau_xiP=.6666, 
-                     tauP.alpha=.001, tauP.beta=.001, 
-                     debug=FALSE, debug2=FALSE, openbugs=TRUE, InitialSeed,
-                     OPENBUGS.directory, WINBUGS.directory ){
+TimeStratPetersenNonDiagError <- function(title,
+                                          prefix,
+                                          time,
+                                          n1,
+                                          m2,
+                                          u2,
+                                          jump.after=NULL,
+                                          logitP.cov=rep(1,length(u2)),
+                                          logitP.fixed=rep(NA,length(u2)),
+                                          n.chains=3,
+                                          n.iter=200000,
+                                          n.burnin=100000,
+                                          n.sims=2000,
+                                          tauU.alpha=1,
+                                          tauU.beta=.05,
+                                          taueU.alpha=1,
+                                          taueU.beta=.05,
+                                          mu_xiP=-2,
+                                          tau_xiP=.6666,
+                                          tauP.alpha=.001,
+                                          tauP.beta=.001,
+                                          debug=FALSE,
+                                          debug2=FALSE,
+                                          InitialSeed){
 #
-#  Fit the smoothed time-Stratified Petersen estimator with NON-Diagonal recoveries 
+#  Fit the smoothed time-Stratified Petersen estimator with NON-Diagonal recoveries
 #  This model allows recoveries outside the stratum of release and error in the smoothed U curve
 #
 #  Packages Required - must be installed BEFORE calling this functin
@@ -33,35 +50,40 @@ TimeStratPetersenNonDiagError <- function(title, prefix, time, n1, m2, u2,
 #  Of the n1 fish released, some are recapturd in this stratum of release (column 1 of m2) or in
 #     subsequent strata (subsequent columns of m2). No fish are assumed to be available for capture
 #     outside the range of strata considered in the matrix of m2
-#  At the same tine, u2 other (unmarked) fish are newly captured in stratum i. 
+#  At the same tine, u2 other (unmarked) fish are newly captured in stratum i.
 #     These EXCLUDE recaptures of marked fish. These are the fish that are "expanded"
-#     to estimate the population size of fish in stratum i.  
-# 
+#     to estimate the population size of fish in stratum i.
+#
 #  Input
 #      prefix - prefix for file name for initial plot of U's
 #      time- the stratum number
 #      n1  - vector of number of fish released in stratum i
 #      m2  - matrix of number of fish recovered who were released in stratum i and recovered in stratum j
 #      u2  - vector of number of unmarked fish captured in stratum i
-#      jump.after - points after which the spline is allowed to jump. Specify as a list of integers in the 
+#      jump.after - points after which the spline is allowed to jump. Specify as a list of integers in the
 #              range of 1:Nstrata. If jump.after[i]=k, then the spline is split between strata k and k+1
 #      logitP.cov - covariates for logit(P)=X beta.logitP.cov
-#                 - specify anything you want for fixed logitP's as the covariate values are simply ignored. 
+#                 - specify anything you want for fixed logitP's as the covariate values are simply ignored.
 #                 - recommend that you specify 1 for the intercept and 0's for everything else
-#      logitP.fixed- values for logitP that are fixed in advance. Use NA if corresponding value is not fixed, 
+#      logitP.fixed- values for logitP that are fixed in advance. Use NA if corresponding value is not fixed,
 #                    otherwise specify the logitP value.
 
 
-#  This routine makes a call to WinBugs to fit the model and then gets back the 
+#  This routine makes a call to WinBugs to fit the model and then gets back the
 #  coda files for the posteriour distribution.
 
-library("R2WinBUGS")  # Make sure that all the packages needed are available
-library("coda")       # used for convergence diagnostics
-library("splines")    # used to compute the Bspline design matrix
+## Set working directory to current directory (we should allow users to select this)
+working.directory <- getwd()
+
+## Define paths for the model, data, and initial value files
+model.file <- file.path(working.directory, "model.txt")
+data.file <- file.path(working.directory,"data.txt")
+init.files <- file.path(working.directory,
+                       paste("inits", 1:n.chains,".txt", sep = ""))
 
 # Save the WinBugs progam to the model.txt file
 #
-sink("model.txt")  # NOTE: NO " allowed in model as this confuses the cat command
+sink(model.file)  # NOTE: NO " allowed in model as this confuses the cat command
 cat("
 model TimeStratPetersenNonDiagError{
 # Time Stratified Petersen with NON Diagonal recapture and allowing for error in the smoothed U curve.
@@ -110,22 +132,30 @@ model TimeStratPetersenNonDiagError{
 #      U[i]
 #       etaU[i]  = log(U[i])
 #         which comes from spline with parameters bU[1... Knots+q]
-#         + error term eU[i] 
+#         + error term eU[i]
 #
 #      muLogTT[i] = mean log(travel time) assuming a log-normal distribution for travel time
 #      sdLogTT[i] = sd   log(travel time) assuming a log-normal distribution for travel time
 #                        Note that the etasdLogTT=log(sdLogTT) is modelled to keep the sd positive
-#       
+#
 
    ##### Fit the spline for the U's and specify hierarchial model for the logit(P)'s ######
    for(i in 1:Nstrata.cap){
-        logUne[i] <- inprod(SplineDesign[i,1:n.bU],bU[1:n.bU])  # spline design matrix * spline coeff 
-        etaU[i] ~ dnorm(logUne[i], taueU)              # add random error
+        logUne[i] <- inprod(SplineDesign[i,1:n.bU],bU[1:n.bU])  # spline design matrix * spline coeff
+        etaU[i] ~ dnorm(logUne[i], taueU)C(,20)              # add random error
         eU[i] <- etaU[i] - logUne[i]
    }
    for(i in 1:Nfree.logitP){   # model the free capture rates using covariates
-        mu.logitP[free.logitP.index[i]] <- inprod(logitP.cov[free.logitP.index[i],1:NlogitP.cov], beta.logitP[1:NlogitP.cov]) 
-        logitP[free.logitP.index[i]] ~ dnorm(mu.logitP[free.logitP.index[i]],tauP)
+        mu.logitP[free.logitP.index[i]] <- inprod(logitP.cov[free.logitP.index[i],1:NlogitP.cov], beta.logitP[1:NlogitP.cov])
+
+
+        ## logitP[free.logitP.index[i]] ~ dnorm(mu.logitP[free.logitP.index[i]],tauP)
+
+        mu.epsilon[free.logitP.index[i]] <- mu.logitP[free.logitP.index[i]] - log(u2[free.logitP.index[i]] + 1) + etaU[free.logitP.index[i]]
+        epsilon[free.logitP.index[i]] ~ dnorm(mu.epsilon[free.logitP.index[i]],tauP)
+
+        logitP[free.logitP.index[i]] <- log(u2[free.logitP.index[i]] + 1) - etaU[free.logitP.index[i]] + epsilon[free.logitP.index[i]]
+
    }
 
 
@@ -236,7 +266,7 @@ SplineDesign <- matrix(0, nrow=0, ncol=0)
 SplineDegree <- 3           # Degree of spline between occasions
 b.flat <- NULL              # index of spline coefficients with a flat prior distribution -first two of each segment
 b.notflat <- NULL           # index of spline coefficients where difference is modelled
-all.knots <- NULL    
+all.knots <- NULL
 for (i in 1:(length(ext.jump)-1)){
   nstrata.in.set <- ext.jump[i+1]-ext.jump[i]
   if(nstrata.in.set > 7)
@@ -246,7 +276,7 @@ for (i in 1:(length(ext.jump)-1)){
     }
   all.knots <- c(all.knots, knots)
   # compute the design matrix for this set of strata
-  z <- bs((1:nstrata.in.set)/(nstrata.in.set+1), knots=knots, degree=SplineDegree, 
+  z <- bs((1:nstrata.in.set)/(nstrata.in.set+1), knots=knots, degree=SplineDegree,
              intercept=TRUE, Boundary.knots=c(0,1))
   # first two elements of b coeffients have a flat prior
   b.flat <- c(b.flat, ncol(SplineDesign)+(1:2))
@@ -261,7 +291,7 @@ n.b.notflat <- length(b.notflat)
 n.bU <- n.b.flat + n.b.notflat
 
 
-# get the logitP covariate matrix ready 
+# get the logitP covariate matrix ready
 logitP.cov <- as.matrix(logitP.cov)
 NlogitP.cov <- ncol(as.matrix(logitP.cov))
 
@@ -272,7 +302,7 @@ free.logitP.index <- (1:Nstrata.cap)[ is.na(logitP.fixed)]  # free values are th
 Nfree.logitP <- length(free.logitP.index)
 
 
-datalist <- list("Nstrata.rel", "Nstrata.cap", "n1", "m2", "u2", 
+datalist <- list("Nstrata.rel", "Nstrata.cap", "n1", "m2", "u2",
                  "logitP", "Nfree.logitP", "free.logitP.index",   # those indices that are fixed and free to vary
                  "logitP.cov", "NlogitP.cov",
                  "SplineDesign",
@@ -281,8 +311,9 @@ datalist <- list("Nstrata.rel", "Nstrata.cap", "n1", "m2", "u2",
                  "mu_xiP", "tau_xiP", "tauP.alpha", "tauP.beta")
 
 
-# get the initial values for the parameters of the model
-
+## Generate best guess initial values
+## These values are only used to draw an initial fit plot and are not
+## used as initial values in MCMC.
 
 Uguess <- pmax(c((u2[1:Nstrata.rel]+1)*(n1+2)/
                  (apply(m2[,1:Nstrata.cap],1,sum)+1),
@@ -306,7 +337,7 @@ if(debug2) {
 
 # create an initial plot of the fit
 pdf(file=paste(prefix,"-initialU.pdf",sep=""))
-plot(time, log(Uguess), 
+plot(time, log(Uguess),
     main=paste(title,"\nInitial spline fit to estimated U[i]"),
     ylab="log(U[i])", xlab='Stratum')  # initial points on log scale.
 lines(time, SplineDesign %*% init.bU)  # add smoothed spline through points
@@ -314,170 +345,89 @@ dev.off()
 
 
 parameters <- c("logitP", "beta.logitP", "tauP", "sigmaP",
-                "bU", "tauU", "sigmaU", 
-                "eU", "taueU", "sigmaeU", 
+                "bU", "tauU", "sigmaU",
+                "eU", "taueU", "sigmaeU",
                 "Ntot", "Utot", "logUne", "etaU", "U",
                  "muLogTT", "sdLogTT")      #  mean and sd of log(travel times)
 if( any(is.na(m2))) {parameters <- c(parameters,"m2")} # monitor in case some bad data where missing values present
 if( any(is.na(u2))) {parameters <- c(parameters,"u2")}
-                 
-init.vals <- function(){
-   init.logitP <- c(logit((apply(m2[,1:Nstrata.cap],1,sum)+1)/(n1+1)),rep(mu_xiP,Nstrata.cap-Nstrata.rel))         # initial capture rates based on observed recaptures
-   init.logitP <- pmin(10,pmax(-10,init.logitP))
-   init.logitP[is.na(init.logitP)] <- -2         # those cases where initial probability is unknown
-   init.logitP[!is.na(logitP.fixed)]<- NA        # no need to initialize the fixed values
-   init.beta.logitP <- as.vector(lm( init.logitP ~ logitP.cov-1)$coefficients)
-   init.beta.logitP[is.na(init.beta.logitP)] <- 0 
-   init.beta.logitP <- c(init.beta.logitP, 0)   # add one extra element so that single beta is still written as a 
-                                             # vector in the init files etc.
-   init.tauP <- 1/var(init.logitP, na.rm=TRUE)     # 1/variance of logit(p)'s (ignoring the covariates for now)
 
-   init.bU   <- lm(log(Uguess) ~ SplineDesign-1)$coefficients  # initial values for spline coefficients
-   init.eU   <- as.vector(log(Uguess)-SplineDesign%*%init.bU)  # error terms set as differ between obs and pred
-   init.etaU <- log(Uguess)
+## init.vals <- function(){
 
-   # variance of spline difference
-   sigmaU <- sd( init.bU[b.notflat]-2*init.bU[b.notflat-1]+init.bU[b.notflat-2], na.rm=TRUE)
-   init.tauU <- 1/sigmaU^2
+##    init.logitP <- c(logit((apply(m2[,1:Nstrata.cap],1,sum)+1)/(n1+1)),rep(mu_xiP,Nstrata.cap-Nstrata.rel))         # initial capture rates based on observed recaptures
+##    init.logitP <- pmin(10,pmax(-10,init.logitP))
+##    init.logitP[is.na(init.logitP)] <- -2         # those cases where initial probability is unknown
+##    init.logitP[!is.na(logitP.fixed)]<- NA        # no need to initialize the fixed values
+##    init.beta.logitP <- as.vector(lm( init.logitP ~ logitP.cov-1)$coefficients)
+##    init.beta.logitP[is.na(init.beta.logitP)] <- 0
+##    init.beta.logitP <- c(init.beta.logitP, 0)   # add one extra element so that single beta is still written as a
+##                                              # vector in the init files etc.
+##    init.tauP <- 1/var(init.logitP, na.rm=TRUE)     # 1/variance of logit(p)'s (ignoring the covariates for now)
 
-   # variance of error in the U over and above the spline fit
-   sigmaeU <- sd(init.eU, na.rm=TRUE)
-   init.taueU <- 1/sigmaeU^2
+##    init.bU   <- lm(log(Uguess) ~ SplineDesign-1)$coefficients  # initial values for spline coefficients
+##    init.eU   <- as.vector(log(Uguess)-SplineDesign%*%init.bU)  # error terms set as differ between obs and pred
+##    init.etaU <- log(Uguess)
 
-   # initialize the u2 where missing
-   init.u2    <- u2
-   init.u2[ is.na(u2)] <- 100
-   init.u2[!is.na(u2)] <- NA
+##    # variance of spline difference
+##    sigmaU <- sd( init.bU[b.notflat]-2*init.bU[b.notflat-1]+init.bU[b.notflat-2], na.rm=TRUE)
+##    init.tauU <- 1/sigmaU^2
 
-   # mean log(travel time) and sd log(travel time)
-   init.muLogTT <- as.vector(log(pmax(
-                   (m2[,1:Nstrata.cap] %*% 1:Nstrata.cap)/(1+apply(m2[,1:Nstrata.cap],1,sum)) - 1:Nstrata.rel,
-                    1, na.rm=TRUE)))
-   init.muLogTT <- as.vector(init.muLogTT)
-   init.etasdLogTT <- log(rep(.5,Nstrata.rel))  # note that log (sd(log travel time)) is being modelled
-   
-   list(logitP=init.logitP, beta.logitP=init.beta.logitP, tauP=init.tauP, 
-        bU=init.bU,  tauU=init.tauU, taueU=init.taueU, etaU=init.etaU, 
-        muLogTT=init.muLogTT, etasdLogTT=init.etasdLogTT)
-}
+##    # variance of error in the U over and above the spline fit
+##    sigmaeU <- sd(init.eU, na.rm=TRUE)
+##    init.taueU <- 1/sigmaeU^2
+
+##    # initialize the u2 where missing
+##    init.u2    <- u2
+##    init.u2[ is.na(u2)] <- 100
+##    init.u2[!is.na(u2)] <- NA
+
+##    # mean log(travel time) and sd log(travel time)
+##    init.muLogTT <- as.vector(log(pmax(
+##                    (m2[,1:Nstrata.cap] %*% 1:Nstrata.cap)/(1+apply(m2[,1:Nstrata.cap],1,sum)) - 1:Nstrata.rel,
+##                     1, na.rm=TRUE)))
+##    init.muLogTT <- as.vector(init.muLogTT)
+##    init.etasdLogTT <- log(rep(.5,Nstrata.rel))  # note that log (sd(log travel time)) is being modelled
+
+##    list(logitP=init.logitP, beta.logitP=init.beta.logitP, tauP=init.tauP,
+##         bU=init.bU,  tauU=init.tauU, taueU=init.taueU, etaU=init.etaU,
+##         muLogTT=init.muLogTT, etasdLogTT=init.etasdLogTT)
+## }
 
 #browser()
 
+## Generate initial values
+init.vals <- genInitVals(model="TSPNDE",
+                         n1=n1,
+                         m2=m2,
+                         u2=u2,
+                         logitP.cov=logitP.cov,
+                         logitP.fixed=logitP.fixed,
+                         SplineDesign=SplineDesign,
+                         n.chains=n.chains)
 
-# set up for the call to WinBugs or OpenBugs
+## Generate data list
+data.list <- list()
+for(i in 1:length(datalist)){
+  data.list[[length(data.list)+1]] <-get(datalist[[i]])
+}
+names(data.list) <- as.list(datalist)
 
-working.directory <- getwd()          # store all data and init files in the current directory
-if(openbugs){
-   cat("\n\n*** Start of call to OpenBugs \n")
-   bugs.directory = OPENBUGS.directory 
-   # the following call sequence is mainly based on the "openbugs" function in the R2WinBugs package
-   modelFile <- "model.txt"  # this was written in the current directory earlier
-   numChains <- n.chains
-   nBurnin   <- n.burnin
-   nIterPostBurnin <- n.iter - n.burnin
-   nThin     <- round(nIterPostBurnin/n.sims)  # want certain # of final samples so decide upon the thining rate
-   over.relax <- FALSE
-   parameters.to.save <- c(parameters, "deviance")  # always get the deviance
-   parametersToSave <- parameters.to.save
+# Set up for the call to WinBUGS or OpenBUGS
 
-   cat("OpenBugs files created in:", working.directory, "\n")
-   BRugs::modelCheck(modelFile)    # check the model
+results <- run.openbugs(modelFile=model.file,
+                        dataFile=data.file,
+                        dataList=data.list,
+                        initFiles=init.files,
+                        initVals=init.vals,
+                        parameters=parameters,
+                        nChains=n.chains,
+                        nIter=n.iter,
+                        nBurnin=n.burnin,
+                        nSims=n.sims,
+                        overRelax=FALSE,
+                        initialSeed=InitialSeed,
+                        working.directory=working.directory,
+                        debug=debug)
 
-   # get the data and write to the data.txt file in the current directory
-   datafileName = file.path(working.directory, "data.txt")
-   data.list <- list()
-   for(i in 1:length(datalist)){
-      data.list[[length(data.list)+1]] <-get(datalist[[i]])
-   }
-   names(data.list) <- as.list(datalist)
-   temp <- BRugs::bugsData(data.list, fileName = datafileName, digits = 5)
-   cat("Data files saved in ", temp, "\n")
-
-   BRugs::modelData(datafileName)
-   cat("Data loaded into model\n")
-   BRugs::modelCompile(numChains)
-
-   # set the random number generator seed
-   BRugs::modelSetSeed(InitialSeed)
-   cat("Random seed initialized with :", InitialSeed, "\n")
-
-   # generate the files to save the initial values
-   initfileNames <- file.path(working.directory, paste("inits", 1:numChains,".txt", sep = ""))
-   inits <- BRugs::bugsInits(inits = init.vals, numChains = numChains, fileName=initfileNames)
-   cat("Initial values generated in ",inits, "\n")
-   BRugs::modelInits(inits)
-   BRugs::modelGenInits()     # generate the initial values for any uninitialized variables
-   cat("Initial values loaded into model\n")
-
-   # now to generate the burnin sample
-   cat("Burnin sampling has been started for ", nBurnin, " iterations.... \n")
-   flush.console()
-   for(iter in seq(1,nBurnin,round(nBurnin/20)+1)){  # generate a report about every 5% of the way
-      cat('... Starting burnin iteration', iter,' which is about ',round(iter/nBurnin*100),"% of the burnin phase at ",date(),"\n")
-      flush.console()
-      BRugs::modelUpdate(round(nBurnin/20)+1, overRelax = over.relax)
-   }
-   cat("Burnin sampling completed \n")
-
-   # generate the non-burnin samples
-   BRugs::dicSet()      # turn on DIC computations
-   on.exit(BRugs::dicClear(), add = TRUE) 
-   cat("DIC collection set \n")  
-   BRugs::samplesSet(parametersToSave)
-   cat("Nodes to monitor set\n")
-   cat("Starting sampling after burnin for ", n.chains," chain each with  a further ", 
-        nIterPostBurnin, " iterations. \n A thining rate of ", nThin, 
-        "will give about ", round(nIterPostBurnin/nThin), " posterior values in each chain... \n")
-   for(iter in seq(1,nIterPostBurnin,round(nIterPostBurnin/20)+1)){
-      cat('... Starting post-burnin iteration', iter,' which is about ',round(iter/nIterPostBurnin*100),"% of the post-burnin phase at ",date(),"\n")
-      flush.console()
-      BRugs::modelUpdate(round(nIterPostBurnin/nThin/20)+1, thin=nThin, overRelax = over.relax) # we do the thining on the fly
-     BRugs::samplesCoda("*", stem=paste(working.directory,"/",sep=""))  # write out coda files for intermediate results
-   }
-   cat("Finished sampling after burnin and thining \n")
-   FinalSeed <- BRugs::modelGetSeed(i=1)
-   cat("Random seed ended with :", FinalSeed, "\n")
- 
-   # Now to extract the sampled values and create the bugs array
-   cat("Extracting the sampled values\n")
-   params <- BRugs::samplesMonitors("*")
-   samples <- sapply(params, BRugs::samplesSample)
-   n.saved.per.chain <- nrow(samples)/numChains
-   samples.array <- array(samples, c(n.saved.per.chain, numChains, ncol(samples)))    
-   dimnames(samples.array)[[3]] <- dimnames(samples)[[2]]
-   DICOutput <- BRugs::dicStats()
-
-   # save the information. The simulation has already been thinned, so no need to thin again
-   results<- as.bugs.array(sims.array = samples.array, 
-        model.file = modelFile, program = "OpenBUGS", DIC = TRUE, 
-        DICOutput = DICOutput, n.iter = n.iter, n.burnin = n.burnin, 
-        n.thin = nThin)
-   results$Seed.initial <- InitialSeed
-   results$Seed.final   <- FinalSeed
-   cat("Final dimension of saved simulation output is ", dim(results$sims.array), "\n")
-
-   # save the information to the coda files
-   BRugs::samplesCoda("*", stem=paste(working.directory,"/",sep=""))  # write out coda files
-   cat("Coda file created \n")
-
-   cat("\n\n*** Finished OpenBugs ***\n\n")
-   results
-   } else {
-   bugs.directory = WINBUGS.directory  
-   results <- bugs( 
-      datalist,   # notice this is a list of external variables 
-      inits=init.vals, # function to generate initial values for each chain
-      parameters,
-      model.file="model.txt",
-      n.chains=n.chains,
-      n.iter=n.iter,    # includes burn in
-      n.burnin=n.burnin,
-      n.sims=n.sims,      # (approx) number of final simulations to save
-      bugs.directory=bugs.directory,
-      working.directory=working.directory,
-      debug=debug     )
-   results
-   } # end of else clause
-
-
-} # end of TimeStratPetersenNonDiagError function
+return(results)
+}
